/**
 * ******************************************************************************
 *  Copyright Fujitsu.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * *****************************************************************************
 *
 *  NIFCLOUD SDK for Java
 *
 */
package com.nifty.cloud.sdk.routetable.model.transform;

import com.nifty.cloud.sdk.routetable.model.Route;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

import java.util.ArrayList;
import java.util.List;

/**
 * RouteSetType変換クラス。<br />
 * このクラスはレスポンスXMLのrouteSet要素からRouteクラスのリストを生成します。
 */
public class RouteSetConverter implements Converter {

	private static final String ELEMENT_DESTINATION_CIDR_BLOCK = "destinationCidrBlock";
	private static final String ELEMENT_GATEWAY_ID = "gatewayId";
	private static final String ELEMENT_INSTANCE_ID = "instanceId";
	private static final String ELEMENT_INSTANCE_OWNER_ID = "instanceOwnerId";
	private static final String ELEMENT_NETWORK_INTERFACE_ID = "networkInterfaceId";
	private static final String ELEMENT_IP_ADDRESS = "ipAddress";
	private static final String ELEMENT_NETWORK_ID = "networkId";
	private static final String ELEMENT_NETWORK_NAME = "networkName";
	private static final String ELEMENT_STATE = "state";
	private static final String ELEMENT_ORIGIN = "origin";
	private static final String ELEMENT_PRIORITY = "priority";
	private static final String ELEMENT_VPC_PEERING_CONNECTION_ID = "vpcPeeringConnectionId";

	/**
	 * 変換可能なクラスか判定します。
	 *
	 * @param clazz クラス
	 * @return 変換可能な場合true, 変換不可能な場合false
	 *
	 * @see com.thoughtworks.xstream.converters.Converter#canConvert(Class)
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public boolean canConvert(Class clazz) {
		Class[] interfaces = clazz.getInterfaces();
		for (Class i : interfaces) {
			if (i == List.class) {
				return true;
			}
		}
		return false;
	}

	/**
	 * {@literal
	 * routeSetをList<Route>クラスに変換します。}
	 *
	 * @param reader XML読込リーダ
	 * @param context
	 * @return {@literal List<Route>オブジェクト}
	 *
	 * @see com.thoughtworks.xstream.converters.Converter#unmarshal(HierarchicalStreamReader, UnmarshallingContext)
	 */
	@Override
	public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
		List<Route> routes = new ArrayList<Route>();
		String name = null;
		while (reader.hasMoreChildren()) {
			reader.moveDown();
			Route route = new Route();

			while (reader.hasMoreChildren()) {
				reader.moveDown();

				name = reader.getNodeName();
				if (name.equals(ELEMENT_DESTINATION_CIDR_BLOCK)) {
					route.setDestinationCidrBlock(reader.getValue());
				} else if (name.equals(ELEMENT_GATEWAY_ID)) {
					route.setGatewayId(reader.getValue());
				} else if (name.equals(ELEMENT_INSTANCE_ID)) {
					route.setInstanceId(reader.getValue());
				} else if (name.equals(ELEMENT_INSTANCE_OWNER_ID)) {
					route.setInstanceOwnerId(reader.getValue());
				} else if (name.equals(ELEMENT_NETWORK_INTERFACE_ID)) {
					route.setNetworkInterfaceId(reader.getValue());
				} else if (name.equals(ELEMENT_IP_ADDRESS)) {
					route.setIpAddress(reader.getValue());
				} else if (name.equals(ELEMENT_NETWORK_ID)) {
					route.setNetworkId(reader.getValue());
				} else if (name.equals(ELEMENT_NETWORK_NAME)) {
					route.setNetworkName(reader.getValue());
				} else if (name.equals(ELEMENT_STATE)) {
					route.setState(reader.getValue());
				} else if (name.equals(ELEMENT_ORIGIN)) {
					route.setOrigin(reader.getValue());
				} else if (name.equals(ELEMENT_PRIORITY)) {
					route.setPriority(reader.getValue());
				} else if (name.equals(ELEMENT_VPC_PEERING_CONNECTION_ID)) {
					route.setVpcPeeringConnectionId(reader.getValue());
				}

				reader.moveUp();
			}
			routes.add(route);
			reader.moveUp();
		}

		return routes;
	}

	/**
	 * 未実装メソッド
	 */
	@Override
	public void marshal(Object paramObject,
			HierarchicalStreamWriter paramHierarchicalStreamWriter,
			MarshallingContext paramMarshallingContext) {
	}
}
