/**
 * ******************************************************************************
 *  Copyright Fujitsu.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * *****************************************************************************
 *
 *  NIFCLOUD SDK for Java
 *
 */
package com.nifty.cloud.sdk.sample;

import static java.lang.System.out;
import static java.lang.System.err;

import java.util.List;

import com.nifty.cloud.sdk.ClientConfiguration;
import com.nifty.cloud.sdk.NiftyClientException;
import com.nifty.cloud.sdk.Protocol;
import com.nifty.cloud.sdk.auth.BasicCredentials;
import com.nifty.cloud.sdk.auth.Credentials;
import com.nifty.cloud.sdk.auth.SignatureUtils.SignatureMethod;
import com.nifty.cloud.sdk.auth.SignatureUtils.SignatureVersion;
import com.nifty.cloud.sdk.usage.model.DescribeUserActivitiesRequest;
import com.nifty.cloud.sdk.usage.model.DescribeUserActivitiesResult;
import com.nifty.cloud.sdk.usage.model.UserActivity;
import com.nifty.cloud.sdk.usage.NiftyUsageClient;

/**
 * DescribeUserActivitiesサンプル
 *
 */
public class DescribeUserActivitiesSample {

    /**
     * DescribeUserActivitiesを実行するサンプルです。
     * 必要なパラメーターを設定してから利用してください。
     *
     * @param args 利用しない
     */
    public static void main(String[] args) {

        /************************************************************************
         * AccessKeyとSecretAccessKeyを設定します。 キーはNIFCLOUDのコントロールパネルから取得します。
         ***********************************************************************/
        String accessKey = "<Input Your AccessKey>";
        String secretKey = "<Input Your SecretAccessKey>";

        /************************************************************************
         * 指定されたAccessKeyとSecretAccessKeyから認証情報インスタンスを生成します。
         ***********************************************************************/
        Credentials credential = new BasicCredentials(accessKey, secretKey);

        /************************************************************************
         * クライアント設定情報インスタンスを生成します。
         * 設定情報を変更する場合は値を設定します。
         * 以下の項目について設定可能です。
         *   - ユーザーエージェント
         *   - プロトコル
         *   - プロキシサーバーアドレス
         *   - プロキシサーバーポート番号
         *   - プロキシユーザー名
         *   - プロキシパスワード
         *   - 最大リトライ回数
         *   - 最大接続数
         *   - TCPソケットタイムアウト秒数
         *   - 接続タイムアウト秒数
         *   - 送信バッファサイズ / 受信バッファサイズ
         *   - SignatureVersion
         *   - SignatureMethod
         *   - HTTPメソッド
         ***********************************************************************/
        ClientConfiguration config = new ClientConfiguration();
        // config.setUserAgent("NIFCLOUD API Java SDK");
        // config.setProtocol(Protocol.HTTPS);
        // config.setProxyHost("proxy_host");
        // config.setProxyPort(0);
        // config.setProxyUsername("proxy_username");
        // config.setProxyPassword("proxy_password");
        // config.setMaxErrorRetry(3);
        // config.setMaxConnections(50);
        // config.setSocketTimeout(30);
        // config.setConnectionTimeout(30);
        // config.setSocketSendBufferSizeHint(0, 0);
        // config.setSignatureVersion(SignatureVersion.Version_2);
        // config.setSignatureMethod(SignatureMethod.HmacSHA256);
        // config.setRequestMethod("GET");

        /************************************************************************
         * 認証情報とクライアント設定情報から利用状況カテゴリ クライアントを生成します。
         * 非同期実行を行う場合はNiftyUsageAsyncClientクラスを指定します。
         ***********************************************************************/
        NiftyUsageClient client = new NiftyUsageClient(credential, config);

        /************************************************************************
         * 接続先URLを変更する場合は接続先を指定します。
         ***********************************************************************/
        // client.setEndpoint("endpoint_url");

        /************************************************************************
         * リクエストパラメーターを設定し、DescribeUserActivitiesを実行します。
         ***********************************************************************/
        DescribeUserActivitiesRequest request = new DescribeUserActivitiesRequest();
        
         // request.setYearMonth("yearMonth");
        
        invokeDescribeUserActivities(client, request);
    }

    /**
     * DescribeUserActivities 実行サンプル。
     * 設定されたリクエストから DescribeUserActivities を実行し、実行結果を表示します。
     * 
     * @param client クライアント
     * @param request リクエスト
     */
    public static void invokeDescribeUserActivities(NiftyUsageClient client,
            DescribeUserActivitiesRequest request) {
        try {
            DescribeUserActivitiesResult result = client.describeUserActivities(request);
            
            out.println("DescribeUserActivities Action Response");
            out.println("=============================================================================");
            
            if (result.getUserActivities() != null) {
                out.println("UserActivities");
                List<UserActivity> userActivities = result.getUserActivities();
                for (UserActivity userActivity : userActivities) {
                    if (userActivity.getDateTime() != null) {
                        out.println("  DateTime     : " + userActivity.getDateTime());
                    }
                    if (userActivity.getIpAddress() != null) {
                        out.println("  IpAddress    : " + userActivity.getIpAddress());
                    }
                    if (userActivity.getCategoryName() != null) {
                        out.println("  CategoryName : " + userActivity.getCategoryName());
                    }
                    if (userActivity.getOperator() != null) {
                        out.println("  Operator     : " + userActivity.getOperator());
                    }
                    if (userActivity.getOperation() != null) {
                        out.println("  Operation    : " + userActivity.getOperation());
                    }
                    if (userActivity.getResult() != null) {
                        out.println("  Result       : " + userActivity.getResult());
                    }
                }
            }
            out.println("=============================================================================");
        } catch(NiftyClientException ex) {
            err.println("Message: " + ex.getMessage());
        }
    }
}
