/**
 * ******************************************************************************
 *  Copyright Fujitsu.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * *****************************************************************************
 *
 *  NIFCLOUD SDK for Java
 *
 */
package com.nifty.cloud.sdk.sample;


import java.util.ArrayList;
import java.util.List;

import com.nifty.cloud.sdk.ClientConfiguration;
import com.nifty.cloud.sdk.NiftyClientException;
import com.nifty.cloud.sdk.Protocol;
import com.nifty.cloud.sdk.auth.BasicCredentials;
import com.nifty.cloud.sdk.auth.Credentials;
import com.nifty.cloud.sdk.auth.SignatureUtils.SignatureMethod;
import com.nifty.cloud.sdk.auth.SignatureUtils.SignatureVersion;
import com.nifty.cloud.sdk.loadbalancer.NiftyLoadBalancerClient;
import com.nifty.cloud.sdk.loadbalancer.model.Listener;
import com.nifty.cloud.sdk.loadbalancer.model.RegisterPortWithLoadBalancerRequest;
import com.nifty.cloud.sdk.loadbalancer.model.RegisterPortWithLoadBalancerResult;

import static java.lang.System.out;
import static java.lang.System.err;

/**
 * RegisterPortWithLoadBalancerサンプル
 *
 */
public class RegisterPortWithLoadBalancerSample {

    /**
     * RegisterPortWithLoadBalancerを実行するサンプルです。
     * 必要なパラメーターを設定してから利用してください。
     *
     * @param args  利用しない
     */
    public static void main(String[] args) {

        /************************************************************************
         * AccessKeyとSecretAccessKeyを設定します。 キーはNIFCLOUDのコントロールパネルから取得します。
         ***********************************************************************/
        String accessKey = "<Input Your AccessKey>";
        String secretKey = "<Input Your SecretAccessKey>";

        /************************************************************************
         * 指定されたAccessKeyとSecretAccessKeyから認証情報インスタンスを生成します。
         ***********************************************************************/
        Credentials credential = new BasicCredentials(accessKey, secretKey);

        /************************************************************************
         * クライアント設定情報インスタンスを生成します。
         * 設定情報を変更する場合は値を設定します。
         * 以下の項目について設定可能です。
         *   - ユーザーエージェント
         *   - プロトコル
         *   - プロキシサーバーアドレス
         *   - プロキシサーバーポート番号
         *   - プロキシユーザー名
         *   - プロキシパスワード
         *   - 最大リトライ回数
         *   - 最大接続数
         *   - TCPソケットタイムアウト秒数
         *   - 接続タイムアウト秒数
         *   - 送信バッファサイズ / 受信バッファサイズ
         *   - SignatureVersion
         *   - SignatureMethod
         *     HTTPメソッド
         ***********************************************************************/
        ClientConfiguration config = new ClientConfiguration();
        // config.setUserAgent("NIFCLOUD API Java SDK");
        // config.setProtocol(Protocol.HTTPS);
        // config.setProxyHost("proxy_host");
        // config.setProxyPort(0);
        // config.setProxyUsername("proxy_username");
        // config.setProxyPassword("proxy_password");
        // config.setMaxErrorRetry(3);
        // config.setMaxConnections(50);
        // config.setSocketTimeout(30);
        // config.setConnectionTimeout(30);
        // config.setSocketSendBufferSizeHint(0, 0);
        // config.setSignatureVersion(SignatureVersion.Version_2);
        // config.setSignatureMethod(SignatureMethod.HmacSHA256);
        // config.setRequestMethod("GET");

        /************************************************************************
         * 認証情報とクライアント設定情報からロードバランサカテゴリ クライアントを生成します。
         * 非同期実行を行う場合はNiftyLoadBalancerAsyncClientクラスを指定します。
         ***********************************************************************/
        NiftyLoadBalancerClient client = new NiftyLoadBalancerClient(
                credential, config);

        /************************************************************************
         * 接続先URLを変更する場合は接続先を指定します。
         ***********************************************************************/
        // client.setEndpoint("endpoint_url");

        /************************************************************************
         * リクエストパラメーターを設定し、RegisterPortWithLoadBalancerを実行します。
         ***********************************************************************/
        RegisterPortWithLoadBalancerRequest request = new RegisterPortWithLoadBalancerRequest();
        // request.setLoadBalancerName("loadBalancerName");
        // List<Listener> listeners = new ArrayList<Listener>();
        // Listener listener = new Listener();
        // listener.setProtocol("protocol");
        // listener.setLoadBalancerPort(0);
        // listener.setInstancePort(0);
        // listener.setBalancingType("balancingType");
        // listeners.add(listener);
        // request.setListeners(listeners);
        invokeRegisterPortWithLoadBalancer(client, request);

    }

    /**
     * RegisterPortWithLoadBalancer 実行サンプル
     * 設定されたリクエストから RegisterPortWithLoadBalancer を実行し、実行結果を表示します。
     *
     * @param client  クライアント
     * @param request リクエスト
     */
    public static void invokeRegisterPortWithLoadBalancer(
            NiftyLoadBalancerClient client,
            RegisterPortWithLoadBalancerRequest request) {
        try {
            RegisterPortWithLoadBalancerResult result = client.registerPortWithLoadBalancer(request);

            out.println("RegisterPortWithLoadBalancer Action Response");
            out.println("=============================================================================");

            if (result.getListeners() != null) {
                List<Listener> listeners = result.getListeners();
                for (Listener listener : listeners) {
                    out.println("Listeners");
                    if (listener.getProtocol() != null) {
                        out.println("  Protocol            : " + listener.getProtocol());
                    }
                    if (listener.getLoadBalancerPort() != null) {
                        out.println("  LoadBalancerPort    : " + listener.getLoadBalancerPort());
                    }
                    if (listener.getInstancePort() != null) {
                        out.println("  InstancePort        : " + listener.getInstancePort());
                    }
                    if (listener.getBalancingType() != null) {
                        out.println("  BalancingType       : " + listener.getBalancingType());
                    }
                }
            }
            out.println("=============================================================================");
        } catch (NiftyClientException ex) {
            err.println("Message: " + ex.getMessage());
        }
    }
}
